(define scene-name "uv")
(display (string-append "Constructing scene " scene-name "...\n"))

(define (diagonalp p)
  (let ((x (frac (uniform-vector-ref p 0)))
	(y (frac (uniform-vector-ref p 1))))
    (or (< (abs (- x y)) 0.1)
	(> (abs (- x y)) 0.9))))

(define chrome-mat (metal (rgb 0.9 0.9 0.95) 0.9 0.9))
(define black-mat (plastic black 0.15 0.2))
(define the-gradient (color-map 0. red
				0.2 yellow
				0.4 green
				0.6 blue
				0.8 purple
				1.0 red))

(define diag-mat (plastic (lambda (ray)
			    (let ((uv (ray-uv ray)))
			      (the-gradient (frac (array-ref uv 0)))))
			  0.1 0.1))

(assq-set! display-options
	   #:infinity
	   (let ((grays (color-map -1.  (rgb 0.2 0.2 0.3)
				    1.  (rgb 0.7 0.7 0.9))))
	     (lambda (ray)
	       (grays (array-ref (ray-direction ray) 2)))))
	     

(define (the-material ray)
  (let ((uv (vscale 3.0 (ray-uv ray)))
	(scale 3))
    (if (diagonalp uv)
	(diag-mat ray)
	(let ((x (frac (* (array-ref uv 0) scale)))
	      (y (frac (* (array-ref uv 1) 2 scale))))
	  (if (or (and (< x 0.5) (< y 0.5))
		  (and (>= x 0.5) (>= y 0.5)))
	      (black-mat ray)
	      (chrome-mat ray))))))

(define bright (rgb 700.0 700.0 700.0))
(define test-source (lightsource-material bright))
(define b (make-point #i(20.0 -20.0 20.0) test-source))

(define p1 (make-transpath (arc #i(0. 0. 0.) 0.5 0.0 pi2)
			   (lambda (x)
			     (let ((f1 (* x pi2 2)))
			       (m* (rotate-x (* x pi2))
				   (translate (dvector (* 1.2 (sin f1)) 1.5 0.0)))))
			   36 100 the-material 0.00001))

(define p2 (copy-primitive p1))
(transform-primitive! p2 (translate #i(4. 0. 0.)))

(define c (list b p1 p2))

(display (string-append "Enclosing " scene-name "...\n"))
(set! c (make-scene c))
(display "Rendering...\n")
(gc)

(define (gentest x y)
  (mktest 320 240 c
	 (dvector 2.5 -0.5 1.4)
	 (dvector 0. 0. 0.)
	 (dvector 1.0 0. 0.)
	 (dvector 0. 0. 1.0)
	 x y))

(define (genimage)
  (render-image c
	 (dvector 7.8 -4.0 4.0)
	 (dvector 3.0 0. 0.)
	 (dvector 1.0 0. 0.)
	 (dvector 0. 0. 1.0)
	 (output-as-ppm)))

(with-output-to-file (string-append scene-name ".ppm") genimage)
(display "Finished.\n")


